Ext.define('Seller.view.commodity.CommoditySub', {
    extend : 'Seller.view.commodity.Commodity',
    alias : 'widget.commoditysub',
    constructor : function() {
        this.callParent(arguments);
    },
    initialize : function(panel, eOpts) {
    },
    /* 增加商品设置数据 */
    setAddData : function() {
        this.optCommViewType = this.ADDVIEW;
        CommodityAddClass.call(this);
        this.initViewContent();
    },
    /* 修改商品设置数据 */
    setEditData : function() {
        this.optCommViewType = this.EDITVIEW;
        CommodityEditClass.call(this);
        this.initViewContent();
    },
    setData : function(viewType) {
        this.optCommViewType = viewType;
        if(this.optCommViewType === this.ADDVIEW) {
            CommodityAddClass.call(this);
        } else {
            CommodityEditClass.call(this);
        }
        this.initViewContent();
    },

});

/**
 * 增加商品类
 * @constructor
 */
function CommodityAddClass() {
    this.initViewContent = function() {
        Ext.getCmp('commodity-toolbar').setTitle('添加商品');
        setTimeout(function() {
            Ext.getCmp('commodity-view-delete').setHidden(true);
        },100);
    };
    this.checkPermission = function() {
        /**
         * 1. 手动打开钱箱
         * 2. 退货
         * 3. 反结账
         * 4. 打折
         * 5. 删除
         * 6. 撤销订单
         * 7. 修改
         * 8. 自定义抹零
         */
        return new Promise(function(resolve, reject) {
            checkPermissions("16", true)
                .then(function() {resolve();})
                .catch(function() {reject();});
        });
        /*if (!checkPermissions("7", true)) {
            return false;
        }
        return true;*/
    };
    this.saveViewData = function(viewData) {
        var me = this;
        viewData.isDelete = '0';
        viewData.syned = '0';
        viewData.status = '1';
        viewData.lastUpdateTime=new Date().format('yyyy-MM-dd hh:mm:ss');
        CommodityDao.checkCommodityByType(viewData.typeId, viewData.name, function(success, datas) {
            if(success && datas) {
                if (datas.length && datas.length > 0) {
                    if (!datas[0].isDelete || (datas[0].isDelete && datas[0].isDelete !== '1')) {
                        showAlert('提示', '已经存在相同名称的商品');
                        me.closeOperator();
                        return;
                    }

                    viewData.isDelete = null;
                    viewData.id = datas[0].id;
                    CommodityDao.updateCommodity(viewData, datas[0].id, function (success) {
                        if (success) {
                            me.afterViewData(viewData);
                        } else {
                            showAlert('提示', '保存出错！');
                            me.closeOperator();
                        }
                    });
                    return;
                }
                viewData.isSupportNorms = '0';
                CommodityDao.addCommodity(viewData, function (success) {
                    if (success) {
                        me.afterViewData(viewData);
                    } else {
                        showAlert('提示', '保存出错！');
                        me.closeOperator();
                    }
                });
            }
        });
    };
    this.afterViewData = function(viewDatas) {
        //this.callParent(arguments);
        this.afterViewDataParent(viewDatas);
    }
};

/**
 * 修改商品类
 * @constructor
 */
function CommodityEditClass() {
    this.initViewContent = function() {
        Ext.getCmp('commodity-toolbar').setTitle(Application.Ftype==Application.SLAVE?"查看商品":'修改商品');
        Ext.getCmp('commodity-view-delete').setHidden(Application.Ftype==Application.SLAVE?true:false);
        setTimePCommAndTypeDisable();
    };
    this.checkPermission = function() {
        /**
         * 1. 手动打开钱箱
         * 2. 退货
         * 3. 反结账
         * 4. 打折
         * 5. 删除
         * 6. 撤销订单
         * 7. 修改
         * 8. 自定义抹零
         */
        return new Promise(function(resolve, reject) {
            checkPermissions("7", true)
                .then(function() {resolve();})
                .catch(function() {reject();});
        });
    },
    this.setRecord = function(record) {
        var commodityForm = Ext.getCmp('commodityForm');
        if (!commodityForm) {
            return;
        }
        var copyRecord = record.copy(record.get('id'));
        copyRecord.set('isTimePrice', copyRecord.get('isTimePrice')=='1'?true:false);
        if(!copyRecord.get('unit') || copyRecord.get('unit') == '')
            copyRecord.set('unit', '例');

        this.commonOperator(copyRecord.getData());
        this.normsOperator(copyRecord.get('isSupportNorms'));
        commodityForm.setRecord(copyRecord);
        this.record = record;
        var parentContainer = Ext.getCmp('commodity-view').parentContainer;

        if (record.get('category') == "2") {
            var categoryList = Ext.getCmp("commodityGridPanel");
            var store = categoryList.getStore();
            store.removeAll();
            if (record.get('comboMealsJson')) {
                var comboMealsJson = record.get('comboMealsJson');
                var comboMeals = Ext.JSON.decode(comboMealsJson)
                for (var ci = 0, clen = comboMeals.length; ci < clen; ci++) {
                    comboMeals[ci].commodityPrice = parseDigitalDecimal(comboMeals[ci].commodityPrice);
                }
                store.add(comboMeals);
                // store.setData(record.get('comboMealsJson'));
            }
        } else {
            if(Ext.getCmp('commodityGridPanel'))
                Ext.getCmp('commodityGridPanel').getStore().removeAll();
        }

        this.comboOperator(record.get('category'));
    },
    this.getSelfViewData = function(viewDatas) {
        if(this.record) {
            viewDatas.labelPrint = this.record.get('labelPrint')==='1' ? '1' : '0';
            //viewDatas.typeId = this.record.get('typeId');
        }
    },
    this.checkNormsCondition = function(viewDatas) { // 检测规格
        var record = Ext.getCmp('commodity-view').record;
        var isSupportNorms = record.get('isSupportNorms');
        if(isSupportNorms == '1') { // 是否支持规格
            for(var normi=1;normi<9;normi++) {
                if(record.get('normsName'+normi) && record.get('normsPrice'+normi)>=0
                    && record.get('normsPrice'+normi) == viewDatas.price) {
                    break;
                }
            }
            if(normi==9) {
                showAlert('提示', '请输入的价格与规格价格之一保持一致!');
                return false;
            }
        }
        return true;
    },
    this.saveViewData = function(viewData) {
        var me = this;
        var record = Ext.getCmp('commodity-view').record;
        viewData.lastUpdateTime=new Date().format('yyyy-MM-dd hh:mm:ss');
        CommodityDao.isSameCommodityName(viewData.typeId, viewData.id, viewData.name, function(success, total) {
            if(success && total > 0) {
                showAlert('提示', '已经存在相同名称的商品!');
                me.closeOperator();
                return;
            }
            // 设置为称重商品后判断当前商品是否存在于套餐内
            if(viewData.isWeight == '1' || (viewData.category == '2' && !!viewData.comboMealsJson)){
                CommodityDao.isExistCombineCommodity(viewData.id, function(success, datas){
                    if(success && datas.length > 0){
                        if(viewData.isWeight == '1'){
                            showAlert('提示', '此商品已经存在于套餐商品内，不能设为称重商品!');
                        }else if(viewData.category == '2'){
                            showAlert('提示', '此商品已经存在于套餐商品内，不能再设置组合套餐!');
                        }
                        me.closeOperator();
                        return;
                    }
                    SellDao.loadSelldetailForComboMeals(function (success, result) {
                        if (success && result.length > 0) {
                            if (viewData.category == '2' || record.get('category') == '2') {
                                var comboDetails = result;
                                var flag = comboDetails.some(function(item){
                                    return item.commodityId === viewData.id;
                                });
                                if (flag) {
                                    showAlert('提示', '此商品已经下单，订单未完成，不能更改套餐信息！');
                                    me.closeOperator();
                                    return;
                                }
                            }
                        }
                        CommodityDao.updateCommodity(viewData, viewData.id, function(success) {
                            if (success) {
                                //showAlert("提示", "修改商品成功");
                                updateComboJsonForCommodity(viewData).then(function() {
                                    me.afterViewData(viewData);
                                });
                                updateOpenSettingForCommodity(viewData);
                                updateSpecialPriceForCommodity(viewData);
                            } else {
                                showAlert('提示', '保存出错！');
                                me.closeOperator();
                            }
                        });
                    });
                });
            }else {
                CommodityDao.updateCommodity(viewData, viewData.id, function(success) {
                    if (success) {
                        //showAlert("提示", "修改商品成功");
                        updateComboJsonForCommodity(viewData).then(function() {
                            me.afterViewData(viewData);
                        });
                        updateOpenSettingForCommodity(viewData);
                        updateSpecialPriceForCommodity(viewData);
                    } else {
                        showAlert('提示', '保存出错！');
                        me.closeOperator();
                    }
                });
            }
        });
    },
    this.afterViewData = function(viewDatas) {
        //this.callParent(arguments);
        this.afterViewDataParent(viewDatas);
    },
    this.deleteCommodity = function() {
        var me = this;
        var commodityId1 = me.record.get("id");
        checkPermissions("5", true).then(function() {
            CommodityDao.isExistCombineCommodity(commodityId1, function(success, res) {
                var comboName = [];// 套餐名字*
                var comboId = ''; // 套餐id
                for (var ri = 0, len = res.length; ri < len; ri++) {
                    var combo = res[ri];
                    comboId = combo.id;
                    comboName[ri] = combo.name;
                }
                if (comboId != commodityId1 && res && res.length > 0) {
                    showAlert("提示", "该商品存在于" + comboName + "套餐内,不能删除！");
                    // showConfirm('提示', '该商品存在于' + comboName + '套餐内,是否删除？', function(buttonId) { // buttonId yes/no*
                    // 	deleteCommodityStatus(buttonId, me);
                    // });
                } else {
                    showConfirm('提示', '确定删除该商品？', function(buttonId) {
                        deleteCommodityStatus(buttonId, me);
                    });
                }
            })
        }).then(function() {});
    },
    this.commodityFilterFn = function() {
        var me = this;
        return function(commInfo) {
            if(!me.record) return commInfo;
            if(!me.record.get('id')) return commInfo;
            if(commInfo.id == me.record.get('id')) return null;
            return commInfo;
        }
    }
}